import { Module, Scope } from '@nestjs/common';
import { RefreshTokenInterceptor, SessionModule } from './core/sessions';
import { AuthModule } from './auth/auth.module';
import { JWTGuard } from './core/guards';
import { APP_FILTER, APP_GUARD, APP_INTERCEPTOR } from '@nestjs/core';
import { HttpExceptionFilter, TransformInterceptor } from './core/response';
import { ApmModule } from './core/apm';
import { CONNECTION_NAME } from './core/strings/constants/base.constants';
import { TypeOrmModule } from '@nestjs/typeorm';
import { ConfigModule } from '@nestjs/config';
import { UserPrivilegeModule } from './modules/user-related/user-privilege/user-privilege.module';
import { UserPrivilegeModel } from './modules/user-related/user-privilege/data/model/user-privilege.model';
import { CqrsModule } from '@nestjs/cqrs';
import { CouchModule } from './modules/configuration/couch/couch.module';

@Module({
  imports: [
    ConfigModule.forRoot({
      isGlobal: true,
    }),
    TypeOrmModule.forRoot({
      name: CONNECTION_NAME.DEFAULT,
      type: 'postgres',
      host: process.env.DEFAULT_DB_HOST,
      port: parseInt(process.env.DEFAULT_DB_PORT),
      username: process.env.DEFAULT_DB_USER,
      password: process.env.DEFAULT_DB_PASS,
      database: process.env.DEFAULT_DB_NAME,
      entities: [
        UserPrivilegeModel,
      ],
      synchronize: true,
    }),
    CqrsModule,
    SessionModule, 
    AuthModule,
    CouchModule,
    
    UserPrivilegeModule,
  ],
  controllers: [],
  providers: [
    /**
     * By default all request from client will protect by JWT
     * if there is some endpoint/function that does'nt require authentication
     * please add `@Unprotected()` decorator to the function
     */
    {
      provide: APP_GUARD,
      scope: Scope.REQUEST,
      useClass: JWTGuard,
    },
    {
      provide: APP_INTERCEPTOR,
      scope: Scope.REQUEST,
      useClass: RefreshTokenInterceptor,
    },
    {
      provide: APP_INTERCEPTOR,
      scope: Scope.REQUEST,
      useClass: TransformInterceptor,
    },
    {
      provide: APP_FILTER,
      scope: Scope.REQUEST,
      useClass: HttpExceptionFilter,
    },
  ],
})
export class AppModule {}
